// Copyright (C) 2008-2012, Software Development Laboratories, "Fish" (David B. Trout)
////////////////////////////////////////////////////////////////////////////////////////
// PipedProcessCtl.h  --  handy class to create process with redirected stdxxx pipes
/////////////////////////////////////////////////////////////////////////////////////////
//
//  Change History:
//
//  04/19/08    1.0.0   Fish    Created.
//  11/06/08    1.0.2   Fish    Use UINT instead of int.
//
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

/////////////////////////////////////////////////////////////////////////////////////////
// PROGRAMMING NOTE: both PIPEBUFSIZE and HOLDBUFSIZE must be an even value in order to
// support UNICODE builds (since wide characters are 2 bytes each). Also, HOLDBUFSIZE
// must be at least 2 bytes (1 wide char) larger than PIPEBUFSIZE, and PIPEBUFSIZE must
// be at least 2 bytes (1 wide char) larger than the "g_pszBufferOverflowMsg" string.

#define  PIPEBUFSIZE        ( 2048 )        // (MUST BE EVEN! TCHARs COULD BE 2 BYTES!)
#define  HOLDBUFSIZE        ( 4096 )        // (MUST BE EVEN! TCHARs COULD BE 2 BYTES!)

typedef void WRITETOSTDXXX( FILE* pStream, UINT nChildId, LPCTSTR pszMsg );

//////////////////////////////////////////////////////////////////////////////////////////
// Custom class to create child process with its stdxxx streams piped back to us...

class CPipedProcessCtl
{
public:

    CPipedProcessCtl        // constructor
    (
        UINT            nChildId,               // child id#
        LPCTSTR         pszCommandLine,         // command line (with arguments)
        LPCTSTR         pszWorkingDirectory,    // desired working directory
        WRITETOSTDXXX*  pfnWriteToStdXXX        // stream o/p function ptr
    );

    virtual ~CPipedProcessCtl();    // destructor
    bool    StartChildProcess();    // (WARNING: NOT REUSABLE!)

    bool    WaitForProcessToExit( DWORD dwTimeOut = INFINITE );

    bool    HasProcessExited   () const { return m_bProcessHasExited; };
    HANDLE  GetProcessHandle   () const { return m_hChildProcess; };
    DWORD   GetProcessId       () const { return m_dwProcessId; };
    DWORD   GetProcessExitCode () const { return m_dwExitCode; };
    UINT    GetChildId         () const { return m_nChildId; };
    LPCTSTR GetCommandLine     () const { return m_strCommandLine; };
    LPCTSTR GetWorkingDirectory() const { return m_strWorkingDirectory; };

protected:

    // The worker threads that monitor the child's stdout/stderr output.
    // They should automatically end whenever they notice the child's
    // stdout/stderr handles are closed.

    CWinThread*  m_pStdoutThread;       // MFC thread ptr
    CWinThread*  m_pStderrThread;       // MFC thread ptr

    friend UINT  StdoutWorkerThread ( void* pParms );
    friend UINT  StderrWorkerThread ( void* pParms );
    friend UINT  MonitorStdXxxPipe  ( DWORD nWhichStdXxx, CPipedProcessCtl* pCtl );

    HANDLE  m_hOurReadFromStdout;       // pvt handle to read data from child's stdout
    HANDLE  m_hOurReadFromStderr;       // pvt handle to read data from child's stderr

    // (helper called by monitor thread; writes complete msgs to proper stdxxx stream)

    void ParseStdXxxMsg( const UINT nChildId, FILE* pStream,
                         LPTSTR pszHoldBuff, UINT& nHoldAmount );

    // (worker thread that monitors child process to detect when it has ended)

    CWinThread*  m_pChildEndedThread;   // MFC thread ptr

    friend UINT  ChildEndedWorkerThread ( void* pParms );

    //////////////////////////////////////////////////////////////////////
    // (class variables)

    UINT     m_nChildId;                // child id#
    CString  m_strCommandLine;          // command line (with arguments)
    CString  m_strWorkingDirectory;     // desired working directory

    HANDLE  m_hChildProcess;            // handle to child process
    DWORD   m_dwProcessId;              // process id# of child process
    DWORD   m_dwExitCode;               // exit code from child process
    bool    m_bProcessHasExited;        // true == child has exited (or never started)

    WRITETOSTDXXX*  m_pfnWriteToStdXXX; // stream o/p function ptr
};

/////////////////////////////////////////////////////////////////////////////////////////
