// Copyright (C) 2008-2012, Software Development Laboratories, "Fish" (David B. Trout)
//////////////////////////////////////////////////////////////////////////////////////////
// StdAfx.h: include file for standard system include files,
// or project specific include files that are used frequently,
// but are otherwise changed relatively infrequently...
//////////////////////////////////////////////////////////////////////////////////////////
//
//  Change History:
//
//  04/19/08    1.0.0   Fish    Created.
//  11/06/08    1.0.2   Fish    Use UINT instead of int.
//  11/15/08    1.0.2   Fish    VS2008 SP1
//  03/17/09    1.1.0   Fish    -rcfile support.
//  08/11/09    1.1.1   Fish    #ifdef _WIN64 #define PROGNAME "RunJobs64"
//  08/20/09    1.1.2   Fish    AssignJobNum function.
//
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TargetVer.h"          // (SoftDevLabs standard)

//////////////////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES        1
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES_COUNT  1

//////////////////////////////////////////////////////////////////////////////////////////
// System PCH #includes...

#include <afx.h>
#include <afxwin.h>                     // MFC core and standard components
#include <afxext.h>                     // MFC extensions
#ifndef _AFX_NO_OLE_SUPPORT
#include <afxdtctl.h>                   // MFC support for Internet Explorer 4 Common Controls
#endif
#ifndef _AFX_NO_AFXCMN_SUPPORT
#include <afxcmn.h>                     // MFC support for Windows Common Controls
#endif
#include <iostream>                     // _ftprintf, etc

//////////////////////////////////////////////////////////////////////////////////////////
// (ensures all modules use same allocator)

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////////////////////////
// Private PCH #includes...

#include "StdioFileEx.h"                // (UNICODE CStdioFile)
#include "Product.h"                    // (product identification)

#define NUM_BYTES( x )                  sizeof ( x )
#define NUM_TCHARS( x )                 ( NUM_BYTES( x ) / NUM_BYTES( TCHAR ) )
#define MAX_CMDLINE_LEN                 ((32 * 1024) - 3)

#pragma hdrstop                         // (stop pre-compiled headers here)

//////////////////////////////////////////////////////////////////////////////////////////
// Project-wide #includes...

#include "resource.h"                   // (resource symbols; may change from time to time)
#include "Version.h"                    // (product version information)
#include "RunJobs.h"                    // (main application class)

//////////////////////////////////////////////////////////////////////////////////////////
// Project-wide constants and variables, etc...

#ifdef _WIN64
  #define  PROGNAME     "RunJobs64"     // (our name, for logging purposes)
#else
  #define  PROGNAME     "RunJobs"       // (our name, for logging purposes)
#endif

#define  JOBNUM_SEMAPHORE_NAME  "RunJobs_JobNum_Semaphore_{CF627680-CE03-4d28-9583-7D7525D6FD8D}"

//////////////////////////////////////////////////////////////////////////////////////////
// (global variables)

extern  CString  g_strProgName;         // (our name as a CString)
extern  CString  g_strCurrDir;          // (current working directory)
extern  DWORD    g_dwMaxJobs;           // (maximum #of simultaneous jobs)
extern  CString  g_strRunJobsFilename;  // (input runjobs filename)
extern  CString  g_strRCFileFilename;   // (output rcfile filename)

//////////////////////////////////////////////////////////////////////////////////////////
// (the following for controlling multiple threads writing to stdout/stderr stream)

extern  CRITICAL_SECTION  g_StdXxxStreamLock;       // (thread syncronization lock)

// Function to handle asynchronous writing to our stdout/stderr stream.
// (used by pipe-monitoring worker threads created by CPipedProcessCtl)

extern  void  WriteToStdXxx( FILE* pStream, UINT nChildId, LPCTSTR pszMsg );

//////////////////////////////////////////////////////////////////////////////////////////
// (global helper functions)

extern CString  FormatLastError ( DWORD dwLastError = GetLastError() );
extern LONG     AssignJobNum    ();

//////////////////////////////////////////////////////////////////////////////////////////
