// Copyright (C) 2008-2012, Software Development Laboratories, "Fish" (David B. Trout)
//////////////////////////////////////////////////////////////////////////////////////////
//
// StdioFileEx.h  --  derived CStdioFile class that transparently handles UNICODE
//
//////////////////////////////////////////////////////////////////////////////////////////
//
// This class is a modified version of code originally
// written by by David Terracino <davet@lycosemail.com>:
//
// "This code is released into the public domain, because
//  it's silly easy. If you want to use and expand it, go
//  ahead; fine by me. If you do expand this class's
//  functionality, please consider sending it back up to
//  the MFC Programmer's Sourcebook at www.codeguru.com.
//
//  And as always, please give credit where credit is
//  due. Don't remove my name from the source."
//
//////////////////////////////////////////////////////////////////////////////////////////
//
// Version 1.1 23 August 2003.   Incorporated fixes from Dennis Jeryd.
// Version 1.3 19 February 2005. Incorporated fixes from Howard J Oh and some of my own.
// Version 1.4 26 February 2005. Fixed stupid screw-up in code from 1.3.
// Version 1.5 18 November 2005. - Incorporated fixes from Andy Goodwin.
//                               - Allows code page to be specified for reading/writing
//                               - Properly calculates multibyte buffer size instead of
//                                  assuming lstrlen(s).
//                               - Should handle UTF8 properly.
//
// Copyright David Pritchard 2003-2005. davidpritchard@ctv.es
//
// You can use this class freely, but please keep my ego happy
// by leaving this comment in place.
//
//////////////////////////////////////////////////////////////////////////////////////////
//
//  Change History:
//
//  11/15/06    1.0.0   Fish    Removed CodePage and GetCharCount crap, remove conversion
//                              crap and use MFC "T2CA", etc, conversion macros instead,
//                              change 'modeWriteUnicode' value to not collide w/MFC.
//
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

//////////////////////////////////////////////////////////////////////////////////////////

class CStdioFileEx: public CStdioFile
{
public:

    static const UINT  modeWriteUnicode;    // Open() flag: write file in UNICODE format

    CStdioFileEx();
    CStdioFileEx( LPCTSTR pszFileName, UINT nOpenFlags );

    virtual BOOL  Open          ( LPCTSTR pszFileName, UINT nOpenFlags, CFileException* pError = NULL );
    virtual BOOL  ReadString    ( CString& rString  );
    virtual void  WriteString   ( LPCTSTR pszString );
    static  bool  IsFileUnicode ( const CString& sFilePath );

    bool IsFileUnicode() const  { return m_bIsUnicodeFile; }

protected:

    bool  m_bIsUnicodeFile;     // (i/p: true == file known to be in UNICODE format)
                                // (o/p: true == file to be written in UNICODE format)

    UINT  m_nOpenFlags;         // (saved from constructor or 'Open' call)

    // (open helper function to modify open flags if needed and set 'm_bIsUnicodeFile')

    UINT  ProcessFlags( const CString& sFilePath, UINT& nOpenFlags );
};

//////////////////////////////////////////////////////////////////////////////////////////
